unit CR.Messaging.Dialogs;

interface

uses
  System.Messaging, System.Classes;

type
  TDlgMessageHandler = class
  private
  strict protected
    procedure DlgMessageConfirm(const Sender: TObject; const M: TMessage); virtual; abstract;
    procedure DlgMessageError(const Sender: TObject; const M: TMessage); virtual; abstract;
    procedure Subscribe;
    procedure Unsubscribe;
  public
    constructor Create;
    destructor Destroy; override;
  end;

  TDlgMessageHandlerLog = class(TDlgMessageHandler)
  private
    FTarget: TStrings;
  strict protected
    procedure DlgMessageConfirm(const Sender: TObject; const M: TMessage); override;
    procedure DlgMessageError(const Sender: TObject; const M: TMessage); override;
  public
    constructor Create(ATarget: TStrings);
    property Target: TStrings read FTarget;
  end;

implementation

uses
  System.SysUtils,
  CR.Messaging;


constructor TDlgMessageHandler.Create;
begin
  inherited;
  Subscribe;
end;

destructor TDlgMessageHandler.Destroy;
begin
  Unsubscribe;
  inherited;
end;

procedure TDlgMessageHandler.Subscribe;
begin
  TDlgMessageError.SubscribeToMessage(DlgMessageError);
  TDlgMessageConfirm.SubscribeToMessage(DlgMessageConfirm);
end;

procedure TDlgMessageHandler.Unsubscribe;
begin
  TDlgMessageError.Unsubscribe(DlgMessageError);
  TDlgMessageConfirm.Unsubscribe(DlgMessageConfirm);
end;

constructor TDlgMessageHandlerLog.Create(ATarget: TStrings);
begin
  inherited Create;
  FTarget := ATarget;
end;

procedure TDlgMessageHandlerLog.DlgMessageConfirm(const Sender: TObject; const M: TMessage);
var
  msg: TDlgMessageConfirm;
begin
  msg := M as TDlgMessageConfirm;
  Target.Add(Format('"%s": %s', [msg.Value, BoolToStr(msg.Confirmed, True)]));
end;

procedure TDlgMessageHandlerLog.DlgMessageError(const Sender: TObject; const M: TMessage);
var
  msg: TDlgMessageError;
begin
  msg := M as TDlgMessageError;
  Target.Add(Format('"%s"', [msg.Value]));
end;

end.
