unit CR.Messaging;

interface

uses
  System.Messaging;

type
  TMessageExt<T> = class(TMessage<T>)
  strict protected
    class function Manager: TMessageManager; virtual;
  public
    class procedure SendMessage(Sender: TObject; const AValue: T);
    class function SubscribeToMessage(const AListenerMethod: TMessageListenerMethod): Integer; overload;
    class procedure Unsubscribe(const AListenerMethod: TMessageListenerMethod; Immediate: Boolean = False); overload;
  end;

type
  TDlgMessageError = class(TMessageExt<string>);

type
  TDlgMessageConfirm = class(TMessageExt<string>)
  private
    FConfirmed: Boolean;
  public
    class function SendMessage(Sender: TObject; const AQuestion: string): Boolean;
    property Confirmed: Boolean read FConfirmed write FConfirmed;
  end;

implementation

class function TMessageExt<T>.Manager: TMessageManager;
begin
  Result := TMessageManager.DefaultManager;
end;

class procedure TMessageExt<T>.SendMessage(Sender: TObject; const AValue: T);
begin
  Manager.SendMessage(Sender, Self.Create(AValue));
end;

class function TMessageExt<T>.SubscribeToMessage(const AListenerMethod: TMessageListenerMethod): Integer;
begin
  Result := Manager.SubscribeToMessage(Self, AListenerMethod);
end;

class procedure TMessageExt<T>.Unsubscribe(const AListenerMethod: TMessageListenerMethod; Immediate: Boolean = False);
begin
  Manager.Unsubscribe(Self, AListenerMethod, Immediate);
end;

class function TDlgMessageConfirm.SendMessage(Sender: TObject; const AQuestion: string): Boolean;
var
  msg: TDlgMessageConfirm;
begin
    msg := TDlgMessageConfirm.Create(AQuestion);
    try
      msg.Confirmed := False;
      Manager.SendMessage(nil, msg, False);
      Result := msg.Confirmed;
    finally
      msg.Free;
    end;
end;

end.
